//----------------------------------------------------------------------------------------
#include "HP03.h"
//----------------------------------------------------------------------------------------


//--------------------------------------------------------------------
// 32kHz PD6
// 
// Die Frequenz errechnet sich:
// 8Mhz(Taktfrequenz)/1(Prescaler)/2(Vollwelle)/32,768khz(Ausgabefrequenz)= 122 (Vergleichswert)
// oC0A PD6
//--------------------------------------------------------------------
void StartMCLK(void)
{
  TCCR0A = 0B01000010;                       // Pin OC0A/PB0 toggeln, WGM01 fr CTC
  TCCR0B = 0B00000001;                       // Prescaler 1
  OCR0A = 122;                               // 32,768khz bei 8MHz (ungenau, da kein Quarz!)
}


//--------------------------------------------------------------------
// ReadCoefficients / calibration data
//--------------------------------------------------------------------
//
// Sensor01              // Sensor02:
//	hp03.c1=27392;        //	hp03.c1= 20203;  0x4EEB
//	hp03.c2=3548;         //	hp03.c2= 4109;   0x100D
//	hp03.c3=347;          //	hp03.c3= 264;    0x0108
//	hp03.c4=482;          //	hp03.c4= 991;    0x03DF
//	hp03.c5=30766;        //	hp03.c5= 34784;  0x87E0
//	hp03.c6=5270;         //	hp03.c6= 5226;   0x146A
//	hp03.c7=2517;         //	hp03.c7= 2500;   0x09C4
//	hp03.a=3;             //	hp03.a= 3;       0x03
//	hp03.b=22;            //	hp03.b= 16;      0x10
//	hp03.c=6;             //	hp03.c= 6;       0x06
//	hp03.d=12;            //	hp03.d= 14;      0x0E
//
//--------------------------------------------------------------------
void HP03_ReadCoefficients(void)
{
  I2C_Init();
  I2C_Start();
  I2C_WriteByte(0xA0);
  I2C_WriteByte(0x10);
  _delay_ms(50);
  I2C_Start();
  I2C_WriteByte(0xA1);
  hp03.c1  = I2C_ReadByte(ack)<<8;
  hp03.c1 += I2C_ReadByte(ack);
  hp03.c2  = I2C_ReadByte(ack)<<8;
  hp03.c2 += I2C_ReadByte(ack);
  hp03.c3  = I2C_ReadByte(ack)<<8;
  hp03.c3 += I2C_ReadByte(ack);
  hp03.c4  = I2C_ReadByte(ack)<<8;
  hp03.c4 += I2C_ReadByte(ack);
  hp03.c5  = I2C_ReadByte(ack)<<8;
  hp03.c5 += I2C_ReadByte(ack);
  hp03.c6  = I2C_ReadByte(ack)<<8;
  hp03.c6 += I2C_ReadByte(ack);
  hp03.c7  = I2C_ReadByte(ack)<<8;
  hp03.c7 += I2C_ReadByte(ack);
  hp03.a   = I2C_ReadByte(ack);
  hp03.b   = I2C_ReadByte(ack);
  hp03.c   = I2C_ReadByte(ack);
  hp03.d   = I2C_ReadByte(nack);       // letztes Byte: kein Acknowledge senden!
  I2C_Stop();
}


//--------------------------------------------------------------------
// ReadTemp
//--------------------------------------------------------------------
unsigned int HP03_ReadTemp(void) {
unsigned int wADT;
unsigned char i1=0,i2=0;

  PORTB |= (1<<PORTB6);      //  HP03D_XCLR ein
  I2C_Start();
  I2C_WriteByte(0xEE);
  I2C_WriteByte(0xFF);
  I2C_WriteByte(0xE8);
  I2C_Stop();

  _delay_ms(40);

  I2C_Start();
  I2C_WriteByte(0xEE);
  I2C_WriteByte(0xFD);
  I2C_Start();

  I2C_WriteByte(0xEF);
  i1 = I2C_ReadByte(ack);
  i2 = I2C_ReadByte(nack);
  I2C_Stop();
  wADT = i1 << 8;
  wADT |= i2;
  PORTB &= ~(1<<PORTB6);     //  HP03D_XCLR aus
  return(wADT);
}


//--------------------------------------------------------------------
// ReadPressure
//--------------------------------------------------------------------
unsigned int HP03_ReadPressure(void) {
unsigned int wADp;
unsigned char p1=0,p2=0;

  PORTB |= (1<<PORTB6);      //  HP03D_XCLR ein
  I2C_Start();
  I2C_WriteByte(0xEE);
  I2C_WriteByte(0xFF);
  I2C_WriteByte(0xF0);
  I2C_Stop();

  _delay_ms(40);

  I2C_Start();
  I2C_WriteByte(0xEE);
  I2C_WriteByte(0xFD);
  I2C_Start();

  I2C_WriteByte(0xEF);
  p1 = I2C_ReadByte(ack);
  p2 = I2C_ReadByte(nack);
  I2C_Stop();

  wADp = p1 << 8;
  wADp |= p2;
  PORTB &= ~(1<<PORTB6);     //  HP03D_XCLR aus
  return(wADp);
}


//--------------------------------------------------------------------
// to do:
//    Hoehenformel:
//                    p(h)=p0*e^(-h/7990)
//                    h die Hhe 
//                    p(h): Druck auf der Hhe h
//                    p0: Druck auf Bodenniveau (also 1,013 bar)
//                    e:  Eulersche Zahl e = 2,718281828459
//                    ^:  Potenzzeichen
//--------------------------------------------------------------------
HP03_DATA hp03_data(void)
{	
unsigned char ab;
signed short dt,offs,sens;
DWORD tmp;

	 HP03_DATA dat;
  tmp = HP03_ReadPressure();
  for (int i=0; i<4; i++)
  {
    tmp += HP03_ReadPressure();
    tmp = tmp>>1;
  }
  hp03.D1 = tmp;
  hp03.D2 = HP03_ReadTemp();

	 dt = hp03.D2-hp03.c5;
	 if (dt>=0) { ab = hp03.a; }
	 else       { ab = hp03.b; }
	 dt-=((signed long)dt*dt*ab)/(signed long)(16384L<<hp03.c);
	 offs=hp03.c2*4+((signed long)((signed short)hp03.c4-1024)*dt)/4096;
	 sens=hp03.c1+((signed long)hp03.c3*dt)/1024;
	 dat.pressure=((signed long)sens*(hp03.D1-7168))/16384-offs;
  // dat.pressure=dat.pressure/2+scale16(hp03.c7,1.6);
	 dat.pressure=((signed long)dat.pressure*10)/32+hp03.c7;            // ohne Hoehenkorrektur

  //dat.temp=6400+((signed long)dt*hp03.c6)/2560-(dt/(1<<(hp03.d-8)));
	 //Temp = 250+DUT*C6/Get2_x(16)-DUT/Get2_x(hp03.d);    

double MiddleData1;
double MiddleData2;
double MiddleData3;
double MiddleData4;

  //DUT = D2-hp03.c5-((D2-hp03.c5)/Get2_x(7))*((D2-hp03.c5)/Get2_x(7))*hp03.b/Get2_x(CC);
  MiddleData1 = (long) hp03.D2-hp03.c5;
  MiddleData2 = MiddleData1*MiddleData1/16384;
  MiddleData3 = MiddleData2*ab;
  MiddleData4 = Get2_x(hp03.c);
  MiddleData4 = MiddleData3/MiddleData4;
  DUT = MiddleData1 - MiddleData4; 

  dat.temp = 250+DUT*hp03.c6/Get2_x(16)-DUT/Get2_x(hp03.d);
  return dat;
}


//---------------------------------------------------
//function: calculate power for 2
//---------------------------------------------------
long int Get2_x(unsigned char i)
{
long int uiData;
  uiData=2;
  i=i-1;
  while(i) {
   uiData <<= 1;
   i--; 
  }
  return uiData;
}


//----------------------------------------------------------------------------------------
//                      E n d   o f   H P 0 3 . c 
//----------------------------------------------------------------------------------------
