//----------------------------------------------------------------------------------------
// I2C / IIC per Software
//----------------------------------------------------------------------------------------
//
// I2C_CLK, I2C_DAT wird per Open Collector gesteuert (DDRxy statt PORTxy):
//
// Init: I2C_CLK, I2C_DAT in DDRx und PORTx 0 setzen (d.h. Tristate, die externen
//       Pull-Ups (z.B. 10KOhm) ziehen dann auf HIGH.
//       Um jetzt die I2C-Clock bzw. Datenleitung nach HIGH zu bewegen, schreib man
//       eine 0 in DDRx ! (NICHT in PORTx !!!)
//
//-----------------------------------------------------------------------------
#include "I2C_SW.h"

//-----------------------------------------------------------------------------
// die folgenden Zeilen bei Aenderung der SCL-,SDA-Pins anpassen:

#define I2C_CLK DDB5     // SCL
#define I2C_DAT DDB3     // SDA
#define I2C_CLK_DDR DDRB
#define I2C_DAT_DDR DDRB

#define I2C_CLK_PORT PORTB
#define I2C_DAT_PORT PORTB

// dazu noch die PINx... Zeile in I2C_ReadByte() und I2C_GetAck() - fertig!
//-----------------------------------------------------------------------------

#define I2C_CLK_HI I2C_CLK_DDR &= ~(1<<I2C_CLK) // 1 ausgeben: DDRx=0 (Eingang, Tristate, gezogen durch PullUp)
#define I2C_CLK_LO I2C_CLK_DDR |=  (1<<I2C_CLK) // 0 ausgeben: DDRx=1 (Ausgang, Portx ist 0, 0 wird ausgegeben)

#define I2C_DAT_HI I2C_DAT_DDR &= ~(1<<I2C_DAT) // 1 ausgeben: DDRx=0 (Eingang, Tristate, gezogen durch PullUp)
#define I2C_DAT_LO I2C_DAT_DDR |=  (1<<I2C_DAT) // 0 ausgeben: DDRx=1 (Ausgang, Portx ist 0, 0 wird ausgegeben)

#define I2C_delay _delay_us(10)                  // Verzoegerung, damit I2C erstmal sicher funktioniert!
                                                 // hier 1. Optimierungsansatz!

//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//  I2C-Init
//-----------------------------------------------------------------------------
void I2C_Init(void) 
{
  I2C_DAT_PORT &= ~(1<<I2C_DAT); 
  I2C_CLK_PORT &= ~(1<<I2C_CLK);

  I2C_CLK_HI;
  I2C_DAT_HI;
  I2C_delay;
  I2C_Stop();
}

//-----------------------------------------------------------------------------
//  I2C-Start
//-----------------------------------------------------------------------------
void I2C_Start(void) 
{
  I2C_DAT_HI;
  I2C_delay;
  I2C_CLK_HI;
  I2C_delay;
  I2C_DAT_LO;
  I2C_delay;
  I2C_CLK_LO;
}


//-----------------------------------------------------------------------------
//  I2C-Stop
//-----------------------------------------------------------------------------
void I2C_Stop(void) 
{
  I2C_CLK_LO;
  I2C_delay;
  I2C_DAT_LO;
  I2C_delay;
  I2C_CLK_HI;
  I2C_delay;
  I2C_DAT_HI;
}


//-----------------------------------------------------------------------------
//  I2C-GetAck   ackstatus 1: Acknowledge vom Slave empfangen
//                         0: kein Acknowledge vom Slave empfangen 
//-----------------------------------------------------------------------------
BYTE I2C_GetAck(void) 
{
BYTE I2C_TimeOutCnt, ackstatus;

  I2C_TimeOutCnt=20;
  I2C_delay;
  I2C_DAT_HI;
  I2C_delay;
  I2C_CLK_HI;
  while ((PINB & (1<<PINB3)) && (I2C_TimeOutCnt>0))     // warten, dass Slave Datenleitung runterzieht
  {                                                     //  oder TimeOut
    I2C_TimeOutCnt--;
    I2C_delay;
  }
  if (I2C_TimeOutCnt>0) { ackstatus = 1; }
  else                  { ackstatus = 0; } 
  I2C_CLK_LO;
  I2C_delay;
  I2C_DAT_HI;
  I2C_delay;
  return ackstatus;
}


//-----------------------------------------------------------------------------
//  I2C-Write a Byte to Slave
//-----------------------------------------------------------------------------
BYTE I2C_WriteByte(BYTE wbyte)
{
BYTE i,s;

  s=0x80;
  I2C_delay;
  for (i=0; i<8; i++)
  {
    if (wbyte & s) { I2C_DAT_HI; } 
    else           { I2C_DAT_LO; }
    s = s>>1;
    I2C_CLK_HI;
    I2C_delay;
    I2C_CLK_LO;
  }
  I2C_delay;
  I2C_DAT_HI;
  I2C_delay;
  i = I2C_GetAck();
  return i;

}


//-----------------------------------------------------------------------------
//  I2C-Read a Byte from Slave
//-----------------------------------------------------------------------------
BYTE I2C_ReadByte(BYTE ak)
{
BYTE i,rbyte,s;

  rbyte = 0;
  s     = 0x80;

  I2C_DAT_HI;
  I2C_delay;
  for (i=0; i<8; i++)
  {
    I2C_delay;
    I2C_CLK_HI;
    I2C_delay;
    if (PINB & (1<<PINB3)) { rbyte |= s; }
    I2C_delay;
    s = s>>1;
    I2C_CLK_LO;
  }
  I2C_DAT_HI;
  I2C_delay;
  if (ak>0) { I2C_DAT_LO; }
  else      { I2C_DAT_HI; }

  I2C_delay;
  I2C_CLK_HI;
  I2C_delay;
  I2C_CLK_LO;
  I2C_DAT_HI;
  I2C_delay;

  return rbyte;
}


//----------------------------------------------------------------------------------------
//                      E n d   o f   I 2 C _ S W . c 
//----------------------------------------------------------------------------------------
