//--------------------------------------------------------------------------------------
// I2C bzw. IIC per Software, da I2C-Hardwarepins anderweitig belegt sind.
//
// Dieses I2C-Beispielprojekt verwendet einen Temp.-/Drucksensor:
// HP03 (temperature and pressure sensor) http://www.hoperf.com/sensor/hp03s.htm
// Andere Komponenten wie LM75, EEPROMS etc. sind natrlich ebenso am I2C-Bus einsetzbar
//--------------------------------------------------------------------------------------
//
// nderungen: 28.03.2010 - adriaNo6
//             Anpassung an BZM Hardware (ATmega168PV 8MHz + Buzzer an PB2)
//             OLED-IIC-Demo SSD1325  (im Geraet lauffaehig)
//             http://home.arcor.de/wehrsdorf/Oled-Display-Recycling.html
//             http://www.mikrocontroller.net/topic/109648
//
// For non-commercial use only!
//
// Umbauten des orginalen BZM:
//
// - fuer Tastennutzung unbedingt den Atmel QFP100 ausbauen
//
// HP03     AtMega168
//  SCL:    PB5 (I2C_SW / ISP_SCK) + ca. 10k PullUp
//  SDA:    PB3 (I2C_SW/ ISP_MOSI) + ca. 10k PullUp
// XCLR:    PB6 (nur bei Messung high)
// MCLK:    PD6 (32kHz)
//
//
// - die Fuse-Bits zur Vollstaendigkeit: !CKDIV8 unprogrammed!
//       efuse = 0x01
//       hfuse = 0xDE
//       lfuse = 0xC2
//
//--------------------------------------------------------------------------------------
#include <avr/sleep.h>
#include <compat/deprecated.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <avr/io.h>
#include <stdint.h>

#include "Screen.h"
#include "HP03.H"
#include "Oled.h"
#include "I2C_SW.h"

//--------------------------------------------------------------------------------------
//#define F_CPU 8000000
#define BaudRate 9600UL //19200UL

//-----------------------------------------------------------------------------
#define SW_SPI   1      // 0: Hardware SPI
                        // 1: Software SPI
//-----------------------------------------------------------------------------
#define ScreenMax 1			    // Anzahl der Screens: 0..ScreenMax

//-----------------------------------------------------------------------------
WORD freemilli=0;
volatile BYTE z001ms=0, z010ms=0, z100ms=0, z01sec=0, z10sec=0, buffer;
volatile BYTE Ovl10ms=0, Ovl100ms=0, Ovl1sec=0, Ovl10sec=0, Ovl1min=0;

volatile BYTE DisplayOnOff = 1;
volatile BYTE ShutdownCnt = 4;        // ShutdownCnt+1min nach letztem Tastendruck: Display
//-----------------------------------------------------------------------------
void Timer1_Init(void);

//void Init_USART(void);
void Uart_Init (void);
void delay_ms(int ms);

void SleepModeOn(void);
void sendc(unsigned char aa);

void uart_puts (char *s);
void usart_putc(unsigned char cc);

void PressRightKey(void);
void PressLeftKey(void);
BYTE CheckKey(void);

//-------------------------------------------------------------------------------------------------
//  - main - main - main - main - main - main - main - main - main - main - main - main - main -
//-------------------------------------------------------------------------------------------------
int main(void)
{
  DDRB  = 0b00000110;
  PORTB = 0b00000000;
    	
  DDRC  = 0b00000110;
  PORTC = 0b00110000;        // PORTC.4 linke Taste / PORTC.5 rechte Taste
  
  DDRD =  0b01110010;
  PORTD = 0b00001000;        // PORTD.3 mittlere Taste  D.6 HP03D

  DisplayOn();               // incl. OledInit();
  //ClearDisplay_fast();
  ClearDisplay();
  Timer1_Init();
  sei(); //Interrupts ein
  StartMCLK();
  _delay_ms(100);
  HP03_ReadCoefficients();

// ------------------------------------------------------------------------------------------------
// - while(1) -- while(1) -- while(1) -- while(1) -- while(1) -- while(1) -- while(1) -- while(1) -
// ------------------------------------------------------------------------------------------------
  while(1)
  { // ------------------------------------------------------------------------
    // zeitgesteuerte Funktionen:
    // ------------------------------------------------------------------------
    //-----------------------10ms----------------------------------------------
    if (Ovl10ms)
    {
      Ovl10ms = 0;
      if (CheckKey()) { DisplayScreen(); }
    }
    //-----------------------100ms---------------------------------------------
    if (Ovl100ms)
    {
      Ovl100ms = 0;
    }
    //-----------------------1s------------------------------------------------
    if (Ovl1sec) 
    {
      Ovl1sec = 0;
      if (activeScreen==0) data = hp03_data();
      DisplayScreen();
    }
    //-----------------------10s-----------------------------------------------
    if (Ovl10sec) 
    {
      Ovl10sec = 0;
    }
    //-----------------------1min----------------------------------------------
    if (Ovl1min) 
    {
      if (ShutdownCnt) { ShutdownCnt--; }
      else             { if (DisplayOnOff) { DisplayOff(); SleepModeOn(); } }
      Ovl1min = 0;
    }
  // --------------------------------------------------------------------------
  }  // --> while(1) {}
  return 0;         // never!
}                   // end of main
//----------------------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Interruptbehandlung: mittlere Taste
//                      Aufwachen aus PowerDown 
//-----------------------------------------------------------------------------
ISR (INT1_vect) 
{
  EIMSK &= ~(1 << INT1);     // External Interrupt Request Diable
  Tastensperre = 30;         // 300ms Tastensperre
  DisplayOn();
};


//----------------------------------------------------------------------------------------
// Initialisierung Timer1A 1ms 8000000/256/1000 = 31,..
//----------------------------------------------------------------------------------------
void Timer1_Init(void) {
  TCCR1A  = 0x00;
  TCCR1B = ((1<<WGM12) | (1<<CS12));   // (CTC - clear counter on match) Prescaler 256
  OCR1A = 31;                          // 1ms 8MHz
  TIMSK1 = (1<<OCIE1A); 
}


//----------------------------------------------------------------------------------------
//
//----------------------------------------------------------------------------------------
ISR(TIMER1_COMPA_vect)
{
  freemilli++;
  z001ms++;
  if (z001ms>=10) { z001ms = 0; z010ms++; Ovl10ms = 1;  }
  if (z010ms>=10) { z010ms = 0; z100ms++; Ovl100ms = 1; }
  if (z100ms>=10) { z100ms = 0; z01sec++; Ovl1sec = 1;  }
  if (z01sec>=10) { z01sec = 0; z10sec++; Ovl10sec = 1; }
  if (z10sec>= 6) { z10sec = 0; Ovl1min = 1;  }
}


//----------------------------------------------------------------------------------------
// Initialisierung USART
//----------------------------------------------------------------------------------------
void Uart_Init(void)
{
  UCSR0B = (0<<RXEN0);
  UBRR0H=(F_CPU/16/BaudRate-1) >> 8;
  UBRR0L=(F_CPU/16/BaudRate-1) & 0xFF;
  UCSR0A=0x00;
  UCSR0B = (1<<RXEN0)|(1<<TXEN0);
  UCSR0C = ((1<<UCSZ01)|(1<<UCSZ00));  
}


//-----------------------------------------------------------------------------
// Delay
//-----------------------------------------------------------------------------
void delay_ms(int ms)
{
int t;
  for (t=0; t<=ms; t++)
  _delay_ms(1);
}


//-----------------------------------------------------------------------------
// SleepMode einschalten -> Aufwachen mit Int1 (mittlere Taste)
//-----------------------------------------------------------------------------
void SleepModeOn(void)
{
  TIMSK0 &= ~(1<<OCIE0A);
  OLED_CS_LO;
  PORTC = 0b00000000;
  DDRC  = 0b00000000;
  EIMSK |= (1 << INT1);                  // externen Interrupt freigeben
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);
  sleep_mode();
}


//----------------------------------------------------------------------------------------
void sendc(unsigned char aa)
{
	while(!(UCSR0A & (1 << UDRE0)));      // wait until UDR ready
	UDR0 = aa;                            // send character
}

//----------------------------------------------------------------------------------------
// Interrupt USART
//----------------------------------------------------------------------------------------
ISR (USART_RX_vect)
{

}


//----------------------------------------------------------------------------------------
// Interrupt USART
//----------------------------------------------------------------------------------------
ISR (USART_TX_vect) 
{

}

//----------------------------------------------------------------------------------------
void uart_puts (char *s)
{
BYTE cc;

	while (*s)                  //  loop until *s != NULL
	{
		cc = *s;
   sendc(cc);
   s++;
	}
}


//-----------------------------------------------------------------------------
// Prueft, ob eine Taste betaetigt wurde (linke, mittlere und rechte Taste!)
//-----------------------------------------------------------------------------
BYTE CheckKey(void)
{
BYTE r = 0;

  if (Tastensperre > 0) { Tastensperre--; }
  else
  {
    if ((PINC & (1<<PINC4))==0)        // linke Taste
    {
      //ShutdownCnt = _30sec;						          // Shutdown-Timer aufziehen!
      //if (!DisplayOnOff) { DisplayOn(); }
      PressLeftKey();
      Tastensperre = 10;                         // 100ms Tastensperre
      Tast_cnt++;
      if (Tast_cnt != Tast_cnt_old) 
      {
        Tast_cnt_old = Tast_cnt;
        //ClearDisplay();      
        r = 1;      
      }
    }

    if ((PIND & (1<<PIND3))==0)
    {
      ShutdownCnt = 4;								            // +1min Shutdown-Timer aufziehen!
      if (!DisplayOnOff) { DisplayOn(); }

      Tast_cnt++;
      if (Tast_cnt>ScreenMax) { Tast_cnt=0; }
      activeScreen = Tast_cnt; 
      Tastensperre = 30;                           // 300ms Tastensperre
      if (Tast_cnt != Tast_cnt_old) 
      {
        Tast_cnt_old = Tast_cnt;
        ClearDisplay();      
        r = 1;      
      }
    }
    
    
    if ((PINC & (1<<PINC5))==0)        // rechte Taste
    {
      //ShutdownCnt = _30sec;								      // Shutdown-Timer aufziehen!
      //if (!DisplayOnOff) { DisplayOn(); }
      PressRightKey();
      Tastensperre = 10;                         // 100ms Tastensperre
      Tast_cnt++;
      if (Tast_cnt != Tast_cnt_old) 
      {
        Tast_cnt_old = Tast_cnt;
        //ClearDisplay();      
        r = 1;      
      }
    }  
  }
  return r;
}


//-----------------------------------------------------------------------------
// rechte Taste
//-----------------------------------------------------------------------------
void PressRightKey(void)
{
  switch (activeScreen)
  {
    case 0:          break;
    case 1:          break;
    case 2: Hoehe++; break;
    case 3:          break;
   default:          break;
  }
}


//-----------------------------------------------------------------------------
// linke Taste
//-----------------------------------------------------------------------------
void PressLeftKey(void)
{
  switch (activeScreen)
  {
    case 0:          break;
    case 1:          break;
    case 2: Hoehe--; break;
    case 3:          break;
   default:          break;
  }
}


//----------------------------------------------------------------------------------------
//                      E n d   o f   m a i n . c 
//----------------------------------------------------------------------------------------
